@@ -5,7 +5,7 @@ Created on April 13,  2022
 @author: wang Haihua
 """
 import pandas as pd
-import pulp
+from pulp import LpProblem, LpMinimize, LpVariable, lpSum, value
 from datetime import datetime
 import numpy as np
 
@@ -325,246 +325,74 @@ def PSO(f, swarm_size=20, max_iter=200, x_min=[-20,-20], x_max=[20,20],
 ###############################     2 DEA                         ###############################
 #################################################################################################
 
-# The core DEA class, setting up and solving the linear programming
-# problems using PuLP.
+def dea(df_input,df_output,model_type='CRS'):
+    ''' Data Evelopment Analysis(DEA) model
 
+    Parameters:
+    ----------
+    - df_input : a DataFrame object containing the input data
+    - df_output: a DataFrame object containing the output data
+    - model_type:
+        - 'CRS': CRS
+        - 'VRS': VRS
 
-class DEAProblem:
+    Yields :
+    --------
+    df_result : the DataFrame of the result
 
-    """
-    A container for the elements of a data envelopment analysis problem. Sets
-    up the linear programmes and solves them with pulp.
-
-    Requires:
-
-        inputs: a pandas dataframe of the inputs to the DMUs
-        outputs: a pandas dataframe of the outputs from the DMUs
-        kind: 'VRS' or 'CRS'
-        in_weights: the weight restriction to apply to all inputs to all DMUs
-                    (default is [0, inf])
-        out_weights: the weight restriction to apply to all outputs to all DMUs
-                     (default is [0, inf)
-
-    Weight restrictions must be specified as a list. To specify only one bound
-    leave the other as None, eg. in_weights=[1, None].
-
-    """
-
-    def __init__(self, inputs, outputs, returns='CRS',
-                 in_weights=[0, None], out_weights=[0, None]):
-        """
-        Set up the DMUs' problems, ready to solve.
-
-        """
-        self.inputs = _to_dataframe(inputs)
-        self.outputs = _to_dataframe(outputs)
-        self.returns = returns
-
-        self.J, self.I = self.inputs.shape  # no of firms, inputs
-        _, self.R = self.outputs.shape  # no of outputs
-        self._i = range(self.I)  # inputs
-        self._r = range(self.R)  # outputs
-        self._j = range(self.J)  # DMUs
-
-        self._in_weights = in_weights  # input weight restrictions
-        self._out_weights = out_weights  # output weight restrictions
-
-        # creates dictionary of pulp.LpProblem objects for the DMUs
-        self.dmus = self._create_problems()
-
-    def _create_problems(self):
-        """
-        Iterate over the inputs and create a dictionary of LP problems, one
-        for each DMU.
-
-        """
-
-        dmu_dict = {}
-        for j0 in self._j:
-            dmu_dict[j0] = self._make_problem(j0)
-        return dmu_dict
-
-    def _make_problem(self, j0):
-        """
-        Create a pulp.LpProblem for a DMU.
-
-        """
-
-        # Set up pulp
-        prob = pulp.LpProblem("".join(["DMU_", str(j0)]), pulp.LpMaximize)
-        self.inputWeights = pulp.LpVariable.dicts("inputWeight", (self._j, self._i),
-                                                  lowBound=self._in_weights[0], upBound=self._in_weights[1])
-        self.outputWeights = pulp.LpVariable.dicts("outputWeight", (self._j, self._r),
-                                                   lowBound=self._out_weights[0], upBound=self._out_weights[1])
-
-        # Set returns to scale
-        if self.returns == "CRS":
-            w = 0
-        elif self.returns == "VRS":
-            w = pulp.LpVariable.dicts("w", (self._j, self._r))
-        else:
-            raise Exception(ValueError)
-
-        # Set up objective function
-        prob += pulp.LpAffineExpression(
-            [(self.outputWeights[j0][r1], self.outputs.values[j0][r1]) for r1 in self._r]) - w
-
-        # Set up constraints
-        prob += pulp.LpAffineExpression([(self.inputWeights[j0][i1],
-                                          self.inputs.values[j0][i1]) for i1 in self._i]) == 1, "Norm_constraint"
-        for j1 in self._j:
-            prob += self._dmu_constraint(j0, j1) - \
-                w <= 0, "".join(["DMU_constraint_", str(j1)])
-        return prob
-
-    def _dmu_constraint(self, j0, j1):
-        """
-        Calculate and return the DMU constraint for a single DMU's LP problem.
-
-        """
-
-        eOut = pulp.LpAffineExpression(
-            [(self.outputWeights[j0][r1], self.outputs.values[j1][r1]) for r1 in self._r])
-        eIn = pulp.LpAffineExpression(
-            [(self.inputWeights[j0][i1], self.inputs.values[j1][i1]) for i1 in self._i])
-        return eOut - eIn
-
-    def _solver(self):
-        """
-        Iterate over the dictionary of DMUs' problems, solve them, and collate
-        the results into a pandas dataframe.
-
-        """
-
-        sol_status = {}
-        sol_weights = {}
-        sol_efficiency = {}
-
-        for ind, problem in list(self.dmus.items()):
-            problem.solve()
-            sol_status[ind] = pulp.LpStatus[problem.status]
-            sol_weights[ind] = {}
-            for v in problem.variables():
-                sol_weights[ind][v.name] = v.varValue
-            sol_efficiency[ind] = pulp.value(problem.objective)
-        return sol_status, sol_efficiency, sol_weights
-
-    def _build_weight_results_dict(self, sol_weights):
-        """
-        Rename weights from input and output column names, then build a
-        pandas dataframe of all weights.
-
-        """
-        import re
-        tmp_dict = {}
-        for dmu, d in list(sol_weights.items()):
-            tmp_dict[dmu] = {}
-            for key, _ in list(d.items()):
-                m = re.search(r'[0-9]+$',key)
-                i = int(m.group(0))
-                if key.startswith("input"):
-                    tmp_dict[dmu]["in_" + str(self.inputs.columns[i])] = d[key]
-                if key.startswith("output"):
-                    tmp_dict[dmu][
-                        "out_" + str(self.outputs.columns[i])] = d[key]
-        weight_results = pd.DataFrame.from_dict(tmp_dict).T
-
-        return weight_results
-
-    def solve(self, sol_type='technical'):
-        """"
-        Solve the problem and create attributes to hold the solutions.
-
-        Takes:
-            sol_type: 'technical'/'allocative'/'economic'
-            dmus: tuple defining range of DMUs to solve for.
-
-        """
-
-        if sol_type == 'technical':
-            sol_status, sol_efficiency, sol_weights = self._solver()
-            weight_results = self._build_weight_results_dict(sol_weights)
-            status_df = pd.Series(sol_status, name='Status')
-            status_df.index = self.inputs.index
-            efficiency_df = pd.Series(sol_efficiency, name='Efficiency')
-            efficiency_df.index = self.inputs.index
-
-            return DEAResults((('Status', status_df),
-                               ('Efficiency', efficiency_df),
-                               ('Weights', weight_results)))
-        else:
-            print("Solution type not yet implemented.")
-            print("Solving for technical efficiency instead.")
-            self.solve()
+    '''    
+    # Set building
+    K = df_input.index
+    I = df_input.columns
+    J = df_output.columns
+    X = df_input.to_dict()
+    Y = df_output.to_dict()
     
+    eta_plus = {}
+    eta_minus={}
+    objective_ = {}
+    eps=0.00000001
+    r = 0
+    
+    for k in K:
+        # Model Building
+        model = LpProblem(model_type+'_'+k, LpMinimize)  
 
-class DEAResults(dict):
+        # Decision variables Building
+        theta_r = LpVariable('theta_r',lowBound=0,upBound=1)
+        lambda_k = LpVariable.dicts('lambda_k', lowBound=0, indexs=K)
+        eta_plus_i = LpVariable.dicts('eta_plus_i',lowBound=0,indexs=I)
+        eta_minus_j = LpVariable.dicts('eta_minus_j',lowBound=0,indexs=J)
 
-    """
-    A class to hold the results of a DEAProblem and provide methods for
-    their examination. Essentially a dictionary of pandas Series with
-    methods for conducting particular operations on DEA results.
+        # Objective Function setting
+        model += theta_r-eps*lpSum(eta_plus_i[i] for i in I)-eps*lpSum(eta_minus_j[j] for j in J)
 
-    """
-
-#    def __init__(self):
-#        super(DEAResults, self).__init__()
-#        pass
-
-    def find_comparators(self, dmu):
-        """
-        Return the DMUs that form the frontier for the specified DMU.
-
-        """
-        pass
-
-    def env_corr(self, env_vars, qq_plot=False):
-        """
-        Determine correlations with environmental/non-discretionary variables
-        using a logit regression. Tobit will be implemented when available
-        upstream in statsmodels.
-
-        Takes:
-            env_vars: A pandas dataframe of environmental variables
-
-        Returns:
-            corr_mod: the statsmodels' model instance containing the inputs
-                      and results from the logit model.
-
-        Note that there can be no spaces in the variables' names.
-        """
-
-        import matplotlib.pyplot as plt
-        from statsmodels.regression.linear_model import OLS
-        from statsmodels.graphics.gofplots import qqplot
-
-        env_data = _to_dataframe(env_vars)
-        corr_data = env_data.join(self['Efficiency'])
-        corr_mod = OLS.from_formula(
-            "Efficiency ~ " + " + ".join(env_vars.columns), corr_data)
-        corr_res = corr_mod.fit()
-
-        #plot qq of residuals
-        if qq_plot:
-            qqplot(corr_res.resid, line='s')
-            plt.title('Distribution of residuals')
-
-        print(corr_res.summary())
-
-        return corr_res
-
-
-def _to_dataframe(indata):
-    """
-    Indexers require input to be a dataframe but the user may pass a
-    series. Check and cast series to dataframes.
-
-    """
-
-    if type(indata) == pd.core.frame.DataFrame:
-        return indata
-    elif type(indata) == pd.core.series.Series:
-        return pd.DataFrame(indata, columns=['input_data'])
-    else:
-        raise TypeError(
-            "Input data is not a valid pandas DataFrame or Series.")
+        # Constraints setting
+        for i in I:
+            model += lpSum([
+                    lambda_k[k] * X[i][k]
+                for k in K]) + eta_plus_i[i] == theta_r * float(X[i][K[r]])
+        for j in J:
+            model += lpSum([
+                    lambda_k[k] * Y[j][k]
+                for k in K]) - eta_minus_j[j] == float(Y[j][K[r]])
+            
+        if model_type.upper()=='VRS':
+            model += lpSum([ lambda_k[k] for k in K]) == 1
+        # Model solving
+        model.solve()
+        
+        eta_plus[k] = {}
+        eta_minus[k] = {}
+        for i in I:
+            eta_plus[k][i] = value(eta_plus_i[i])
+        for j in J:
+            eta_minus[k][j] = value(eta_minus_j[j])
+        objective_[k] = value(model.objective)
+        
+        r += 1
+    df_obj = pd.DataFrame(pd.Series(objective_),columns=['Efficency']).T
+    df_plus = pd.DataFrame(eta_plus)
+    df_minus = pd.DataFrame(eta_minus)
+    df_result = pd.concat([df_obj,df_plus,df_minus]).T
+    return df_result
